#!/usr/bin/env perl
#
#  elmo - ELectronic Mail Operator
#
#  Copyright (C) 2003 Krzysztof Gibas <duparyba % wp.pl>
#                     rzyjontko
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2, or (at your option)
#  any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
#
#  ------------------------------------------------------------
#
#  This script sets up user's config file based on few simple questions.

use strict;
use warnings;

#set secure environment
$ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin:/usr/pkg/bin';
delete @ENV{'IFS', 'CDPATH', 'ENV', 'BASH_ENV'};

my ($root, $ok, $start, $trash, $drafts, $sent, $spam, $localbox, $bayes_file, 
    $guess_email, $editor, $charset, $addressbook, $signature, $smtp_auth_user, 
    $smtp_auth_passwd);

# This function takes 2 arguments.  First is the question (without trailing
# space or ask sign).  Second is the default value.  User will be shown the
# question and asked for a response.  If no answer is supplied, the default
# one will be returned.
sub ask
{
    my $question = shift @_;
    my $default  = shift @_;
    print "$question [$default]: ";
    my $response = <STDIN>;
    chop $response;
    if ($response eq "" or $response =~ /^(\s+|\t+)$/){
	return $default;
    }
    return $response;
}

# Takes only 1 argument and replaces leading tilde (~) with users home
# directory.
sub replace_tilde
{
    $_ = shift @_;
    s/^~/$ENV{HOME}/g;
    return $_;
}


sub have_space
{
    $_ = shift @_;
    if (/.*\s+.*/ || length($_) == 0) {
        return "\'$_\'";
    }
    else {
        return $_;
    }
}

sub is_valid_maildir
{
    my $dir = shift @_;
    return -d $dir && -d "$dir/cur" && -d "$dir/new" && -d "$dir/tmp";
}


sub make_maildir
{
    my $dir = shift @_;
    if (-f $dir){
        unlink ($dir) || die "\nCouldn't delete file: $dir\n";
    }
    if (-d $dir){
	system ('rm', '-rf', "$dir") || die "\nCouldn't delete directory: $dir\n";
    }
    
    mkdir ("$dir", 0700) || die "\nCouldn't create $dir : $!\n";
    mkdir ("$dir/new", 0700) || die "\nCouldn't create $dir/new : $!\n";
    mkdir ("$dir/cur", 0700) || die "\nCouldn't create $dir/cur : $!\n";
    mkdir ("$dir/tmp", 0700) || die "\nCouldn't create $dir/tmp : $!\n";
}


sub print_valid
{
    my $dir = shift @_;
    print "\nThe directory $dir seems to be a valid maildir.\n";
    print "Using it as a $dir directory.\n\n";
}

sub print_dir_exists
{
    my $dir = shift @_;
    print "\nThere is a $dir directory under the specified directory,\n";
    print "but it is not a valid maildir. Maybe another program is using\n";
    print "this directory to store your messages, but using different\n";
    print "format. You will be asked for a box. If $dir is your answer,\n";
    print "it will be destroyed.\n\n"
}

sub print_file_exists
{
    my $file = shift @_;
    print "\nThere is a file $file under the specified directory.\n";
    print "You can't use it as a $file because elmo won't be able to\n";
    print "move messages there. You will be asked for a box. If $file\n";
    print "is your answer, it will be removed.\n\n";
}

sub print_only_name
{
    print "\nPlease specify only a name of the box under the specified\n";
    print "directory. If it doesn't exist, it will be created for\n";
    print "you. This name must be relative and cannot contain / sign.\n\n";
}


sub local_box
{
    my $localbox = shift @_;
    print "\nIt seems that you have a local spool mailbox located at\n";
    print "$localbox. You can read mail delivered to $localbox\n";
    print "with elmo, but there must be a symbolic link created.\n\n";
}

# STEP 1
# Set up root directory of mailbox.
do {
    $root = ask ("Where do you want to store your mailboxes", "~/mail");
    $root = replace_tilde ($root);
    if (-d $root){
	$ok = 1;
    }
    elsif (! -e $root){
	mkdir ($root, 0700) || die "\nCouldn't create $root : $!\n";
	$ok = 1;
    }
    else {
	print "\nYou have specified an existing file to be a location\n";
	print "of your mailboxes collection. I'm afraid it's impossible.\n";
	print "Elmo stores your mailboxes in a single directory.\n\n";
	print "Please specify a directory name.\n\n"
    }
} while (! $ok);


# STEP 2
# Start box.
if (is_valid_maildir ("$root/inbox")){
    print_valid ("inbox");
    $start = "inbox";
}
elsif (-d "$root/inbox"){
    print_dir_exists ("inbox");
    $start = ask ("What is your start box", "inbox");
    make_maildir ("$root/$start");
}
elsif (-f "$root/inbox"){
    print_file_exists ("inbox");
    $start = ask ("What is your start box", "inbox");
    make_maildir ("$root/$start");
}
else {
    $start = ask ("What is your start box", "inbox");
    while (index ($start, "/") != -1){
	print_only_name();
	$start = ask ("What is your start box", "inbox");
    }
    make_maildir ("$root/$start");
}

# STEP 3
# Trash.
if (is_valid_maildir ("$root/trash")){
    print_valid ("trash");    
    $trash = "trash";
}
elsif (-d "$root/trash"){
    print_dir_exists ("trash");
    $trash = ask ("Where do you want to move deleted messages", "trash");
    make_maildir ("$root/$trash");
}
elsif (-f "$root/trash"){
    print_file_exists ("trash");
    $trash = ask ("Where do you want to move deleted messages", "trash");
    make_maildir ("$root/$trash");
}
else {
    $trash = ask ("Where do you want to move deleted messages", "trash");
    while (index ($trash, "/") != -1){
	print_only_name();
	$trash = ask ("Where do you want to move deleted messages", "trash");
    }
    make_maildir ("$root/$trash");
}

# STEP 4
# Drafts.
if (is_valid_maildir ("$root/drafts")){
    print_valid ("drafts");
    $drafts = "drafts";
}
elsif (-d "$root/drafts"){
    print_dir_exists ("drafts");
    $drafts = ask ("Where do you want to store your unfinished messages", "drafts");
    make_maildir ("$root/$drafts");
}
elsif (-f "$root/drafts"){
    print_file_exists ("drafts");
    $drafts = ask ("Where do you want to store your unfinished messages", "drafts");
    make_maildir ("$root/$drafts");
}
else {
    $drafts = ask ("Where do you want to store your unfinished messages", "drafts");
    while (index ($drafts, "/") != -1){
	print_only_name();
	$drafts = ask ("Where do you want to store your unfinished messages", "drafts");
    }
    make_maildir ("$root/$drafts");
}

# STEP 5
# Sent.
if (is_valid_maildir ("$root/sent")){
    print_valid ("sent");
    $sent = "sent";
}
elsif (-d "$root/sent"){
    print_dir_exists ("sent");
    $sent = ask ("Where do you want to store your sent messages", "sent");
    make_maildir ("$root/$sent");
}
elsif (-f "$root/sent"){
    print_file_exists ("sent");
    $sent = ask ("Where do you want to store your sent messages", "sent");
    make_maildir ("$root/$sent");
}
else {
    $sent = ask ("Where do you want to store your sent messages", "sent");
    while (index ($sent, "/") != -1){
	print_only_name();
    	$sent = ask ("Where do you want to store your sent messages", "sent");
    }
    make_maildir ("$root/$sent");
}

# STEP 6
# Spam.
if (is_valid_maildir ("$root/spam")){
    print_valid ("spam");
    $spam = "spam";
}
elsif (-d "$root/spam"){
    print_dir_exists ("spam");
    $spam = ask ("Where do you want to store your spam messages", "spam");
    make_maildir ("$root/$spam");
}
elsif (-f "$root/spam"){
    print_file_exists ("spam");
    $spam = ask ("Where do you want to store your spam messages", "spam");
    make_maildir ("$root/$spam");
}
else {
    $spam = ask ("Where do you want to store your spam messages", "spam");
    while (index ($spam, "/") != -1){
	print_only_name();
	$spam = ask ("Where do you want to store your sent messages", "sent");
    }
    make_maildir ("$root/$spam");
}

# STEP 6a
# Existing spool mailboxes
if ($ENV{MAILDIR} && -e $ENV{MAILDIR}){
    local_box ($ENV{MAILDIR});
    $localbox = ask ("How do you want to name your local box", "local");
    symlink ("$ENV{MAILDIR}", "$root/$localbox");
}
if ($ENV{MAIL} && -e $ENV{MAIL}){
    local_box ($ENV{MAIL});
    $localbox = ask ("How do you want to name your local box", "local");
    symlink ("$ENV{MAIL}", "$root/$localbox");
}


# STEP 7
# SMTP
if ($ENV{EMAIL}){
    $guess_email = $ENV{EMAIL};
}
else {
    $guess_email = "$ENV{USER}\@$ENV{HOSTNAME}";
}
my $my_name     = ask ("What is your name", "$ENV{USER}");
my $email       = ask ("What is your email", "$guess_email");
my $smtp_server = ask ("What is your smtp server", "$ENV{HOSTNAME}");

my $smtp_auth   = ask ("Should I use smtp auth for your account", "yes");

if ($smtp_auth eq "yes") {
    $smtp_auth_user  = ask ("What is your smtp auth username", "$ENV{USER}");
    $smtp_auth_passwd = ask ("What is your smtp auth password", "");
}

# STEP 8
# POP3

if ($smtp_auth ne "yes") {
    $smtp_auth_user = $ENV{USER};
    $smtp_auth_passwd = "";
}

my $pop_server = ask ("What is your pop3 server", "$ENV{HOSTNAME}");
my $username   = ask ("What is your pop3 username", "$smtp_auth_user");
my $password   = ask ("What is your pop3 password", "$smtp_auth_passwd");

# STEP 9
# SPAM PROTECTION
my $protect     = ask ("Should I protect your mail against spam", "yes");

if ($protect eq "yes"){
    $bayes_file  = "$root/.bayes_file";
}

# STEP 10
# ADDRESSBOOK
$addressbook  = ask ("Where do you want store your addressbook", "$root/.addressbook");
$addressbook = replace_tilde($addressbook);

open(ADDRESSBOOK, ">$addressbook") || print "\nCouldn't create addressbook: $!\n";
close(ADDRESSBOOK) || print "\nCouldn't close $addressbook: $!\n";

# STEP 11
# EDITOR
if ($ENV{EDITOR}){
  $editor  = ask ("What is your favorite editor", "$ENV{EDITOR}");
}
else{
  $editor  = ask ("What is your favorite editor", "vim");
}
  
# STEP 12
# SIGNATURE
$signature = ask ("Where is your signature file", "$ENV{HOME}/.signature");
$signature = replace_tilde($signature);

# STEP 13
# CHARSET

my %LANG = (
    "ca_ES" => "iso-8859-1",
    "cs_CZ" => "iso-8859-2",
    "da_DK" => "iso-8859-1",
    "de_DE" => "iso-8859-1",
    "el_GR" => "iso-8859-7",
    "es_ES" => "iso-8859-1",
    "et_EE" => "iso-8859-1",
    "fi_FI" => "iso-8859-1",
    "fr_FR" => "iso-8859-1",
    "gl_ES" => "iso-8859-1",
    "he_IL" => "iso-8859-8",
    "hr_HR" => "iso-8859-2",
    "hu_HU" => "iso-8859-2",
    "is_IS" => "iso-8859-1",
    "it_IT" => "iso-8859-1",
    "lt_LT" => "iso-8859-13",
    "nl_NL" => "iso-8859-1",
    "nn_NO" => "iso-8859-1",
    "no_NO" => "iso-8859-1",
    "pl_PL" => "iso-8859-2",
    "pt_PT" => "iso-8859-1",
    "ro_RO" => "iso-8859-2",
    "ru_RU" => "iso-8859-5",
    "sk_SK" => "iso-8859-2",
    "sl_SI" => "iso-8859-2",
    "sv_SE" => "iso-8859-1",
    "tr_TR" => "iso-8859-9"
);


if (($ENV{LANG} && $ENV{LANG} eq 'C') || ($ENV{LC_ALL} && $ENV{LC_ALL} eq 'C') || ($ENV{LC_CTYPE} && $ENV{LC_CTYPE} eq 'C')) {
    $charset = "US-ASCII";
}
elsif ($ENV{LC_ALL} and exists ($LANG{"$ENV{LC_ALL}"})) {
    $charset = $LANG{"$ENV{LC_ALL}"};
}
elsif ($ENV{LC_CTYPE} and exists ($LANG{"$ENV{LC_CTYPE}"})) {
    $charset = $LANG{"$ENV{LC_CTYPE}"};
}
elsif ($ENV{LANG} and exists ($LANG{"$ENV{LANG}"})) {
    $charset = $LANG{"$ENV{LANG}"};
}
else {
    $charset = ask ("What is your charset", "iso-8859-1");
}

if (! -d "$root/.pop"){
    mkdir ("$root/.pop", 0700) || print "\nCouldn't create pop_info_dir - $root/.pop: $!\n";
}
my $pop_info_dir = "$root/.pop";

    
if (-e "$ENV{HOME}/.elmorc"){
    print "\nFound existing .elmorc in your home directory. The existing\n";
    print "file will be renamed to .elmorc.0.\n\n";
    rename "$ENV{HOME}/.elmorc", "$ENV{HOME}/.elmorc.0";
}


open (CONFIG, ">$ENV{HOME}/.elmorc") || die "\nCouldn't create $ENV{HOME}/.elmorc: $!\n";
flock (CONFIG, 2) || die "\nCouldn't lock file $ENV{HOME}/.elmorc: $!";

print CONFIG "set mailbox {\n  name: 'my box'\n  root: " . have_space($root) . "\n";
print CONFIG "  start: " . have_space($start) . "\n  drafts: " . have_space($drafts) . "\n";
print CONFIG "  trash: " . have_space($trash) . "\n  sent: " . have_space($sent) . "\n";
print CONFIG "  spam: " . have_space($spam) . "\n  protect: " . have_space($protect) . "\n}\n\n";

if ($bayes_file and $protect eq "yes") {
    print CONFIG "set bayes_file " . have_space($bayes_file) . "\n";
}

print CONFIG "set addressbook " . have_space($addressbook) . "\n";
print CONFIG "set sigfile " . have_space($signature) . "\n";
print CONFIG "set editor \'$editor \%f\'\n";
print CONFIG "set charset " . have_space($charset) . "\n";
print CONFIG "set indent_string '> '\n";
print CONFIG "set sig_dashes on\n";
print CONFIG "#set welcome_msg no\n";
print CONFIG "set relative_names on\n\n";

print CONFIG "set line_format \'\%\?\%\$\%\# \%D \%016f \(\%\-06S\)  \'\n";
print CONFIG "set sent_format \'\%\?\%\$  \%D \%016t \(\%-06S\)  \'\n";
print CONFIG "set abook_format \'\%s \%020n \%e\'\n";
print CONFIG "set fetch_format \'\%\?\%\$  \%D \%016f \(\%-06S\)  \%s\'\n";
print CONFIG "set help_format \'\%012k \%030f \%d\'\n\n";

print CONFIG "set smtp_acc {\n  name: $smtp_server\n  server: $smtp_server\n";
print CONFIG "  my_name: " . have_space($my_name) . "\n  email: $email\n  port: 25\n";

if ($smtp_auth eq "yes") {
    print CONFIG "  username: " . have_space($smtp_auth_user) . "\n";
    print CONFIG "  password: " . have_space($smtp_auth_passwd) . "\n";
}

print CONFIG "\}\n\n";

print CONFIG "set pop_acc {\n  name: $pop_server\n  server: $pop_server\n";
print CONFIG "  username: " . have_space($username) . "\n  password: " . have_space($password) . "\n  port: 110\n";
print CONFIG "  count_max: 0\n}\n\n";

print CONFIG "set pop_info_dir " . have_space($pop_info_dir) . "\n\n";

# KEY BINDINGS
print CONFIG "#KEY BINDINGS\n";
print CONFIG "key folder \\Cn folder_bar_next\n";
print CONFIG "key folder \\Cp folder_bar_prev\n";
print CONFIG "key mail r sender_open_reply\n";
print CONFIG "key mail f sender_open_fwd\n";
print CONFIG "key mail R sender_open_reply_all\n";
print CONFIG "key folder <f1> help_open\n";
print CONFIG "key help   q    help_close\n";
print CONFIG "key select \\Cg  box_selection_leave\n\n";

# HOOKS
print CONFIG "#HOOKS\n";
print CONFIG "hook folder_toggle_flag folder_bar_next\n";
print CONFIG "hook fetch_del_mail fetch_next\n";
print CONFIG "hook fetch_get_mail fetch_next\n";
print CONFIG "hook mybox_create_box box_selection_re_read\n\n";

close (CONFIG) || print "\nCouldn't close file: $!\n";

chmod (0600, "$ENV{HOME}/.elmorc") or die "\nCouldn't chmod $ENV{HOME}/.elmorc: $!\n";

